/**
 * @file       adc.h
 * @brief      Analog To Digital Converter interface for AT90S4433
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/25
 */

#ifndef __ADC_H__
#define __ADC_H__

#include "util.h"

#define ADC_SINGLE_CONVERSION 0
#define ADC_FREE_RUN          1

#define ADC_ENABLE_INTERRUPT  1
#define ADC_DISABLE_INTERRUPT 0

#define ADC_CONVERTING      0
#define ADC_CONVERSION_DONE 1

/**
 * @brief Initialize the ADC
 * @param freeMode if is set to ADC_SINGLE_CONVERSION, the ADC performs only 
 *        one conversion at a time. adcStart must be run to start each 
 *        conversion. \n
 *        If set to ADC_FREE_RUN, the ADC performs continuous conversions.
 * @param enableInterrupt if is set to ADC_ENABLE_INTERRUPT, after each 
 *        conversion the INTERRUPT(SIG_ADC)  is raised and you must have 
 *        implemented this function.  \n
 *        If set to ADC_DISABLE_INTERRUPT there are no interrupt and you can 
 *        get the status of the last conversion from adcIsConversionDone.
 * @see adcStart, adcIsConversionDone
 */
void adcInit(unsigned char freeRun,
			 unsigned char enableInterrupt);

/**
 * @brief Start/stop the ADC conversion of all the adc pins
 * @param mode TRUE run continuous pin measurement \n
 *             FALSE run continuous pin measurement
 */
void adcAutomaticMeasurement(Boolean mode);

/**
 * @brief return the last measure of a pin
 * @param pin Id of the pin [0..5]
 * @return last analogic value of the pin
 */
unsigned char adcGetValue(unsigned char pin);

/**
 * @brief Register a callback that will be run after processing each 'adc read'
 */
void adcRegisterTreatmentCB(UtilFnPtr callback);

/**
 * @brief Return the callback that will be run after processing each 'adc read'
 */
UtilFnPtr adcGetTreatmentCB();

#endif /* __ADC_H__ */
