/**
 * @file       debug.c
 * @brief      Interface with remote PC
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/29
 */

#ifdef DEBUG
#include "debug.h"

#include "util.h"
#include "uart.h"
#include "adc.h"
#include "servo.h" 
#include "motor.h"
#include "bumper.h"
#include "led.h"
#include "wheel.h"
#include "solarDir.h"

/* ------------------------------------------------------------------------ */
/* debugParse                                                               */
/* ------------------------------------------------------------------------ */
void debugParse(unsigned char msg)
{
	unsigned char data1=0, data2=0;
	switch(msg) {
		case DEBUG_PING:
			uartWrite(DEBUG_PING);
			break;
		case DEBUG_LED_OFF:
			ledSetMode(uartRead(UART_WAIT_RECEIVE), LED_MODE_OFF);
			break;
		case DEBUG_LED_BLINK_SLOW:
			ledSetMode(uartRead(UART_WAIT_RECEIVE), LED_MODE_BLINK_SLOW);
			break;
		case DEBUG_LED_BLINK_FAST:
			ledSetMode(uartRead(UART_WAIT_RECEIVE), LED_MODE_BLINK_FAST);
			break;
		case DEBUG_LED_ON:
			ledSetMode(uartRead(UART_WAIT_RECEIVE), LED_MODE_ON);
			break;
		case DEBUG_SERVO_SET:
			data1=uartRead(UART_WAIT_RECEIVE);
			data2=uartRead(UART_WAIT_RECEIVE);
			servoSetPosition(data1, data2);
			break;
		case DEBUG_SERVO_INC:
			servoPosInc(uartRead(UART_WAIT_RECEIVE));
			break;
		case DEBUG_SERVO_DEC:
			servoPosDec(uartRead(UART_WAIT_RECEIVE));
			break;
		case DEBUG_SERVO_DISABLE:
			servoDisable(uartRead(UART_WAIT_RECEIVE));
			break;
		case DEBUG_SERVO_DISABLE_ALL:
			servoDisableAll();
			break;
		case DEBUG_MOTOR_ON:
			motorOn();
			break;
		case DEBUG_MOTOR_OFF:
			motorOff();
			break;
		case DEBUG_MOTOR_SPEED:
			motorSetSpeed(uartRead(UART_WAIT_RECEIVE));
			break;
		case DEBUG_BUMPER_GET:
			uartWrite(bumperGetData());
			break;
		case DEBUG_ADC_GET:
		    {
				adcAutomaticMeasurement(TRUE);
				adcAutomaticMeasurement(FALSE);
			}
			break;
		case DEBUG_SOLAR_GET: 
			solarDump();
			break;
		case DEBUG_WHEEL_STOP:
			wheelStop();
			break;
		case DEBUG_WHEEL_FORWARD:
			wheelForward();
			break;
		case DEBUG_WHEEL_BACKWARD:
			wheelBackward();
			break;
		case DEBUG_WHEEL_LEFT:
			wheelRotateLeft();
			break;
		case DEBUG_WHEEL_RIGHT:
			wheelRotateRight();
			break;
		default:
			uartWrite(0xFF);
			uartWrite(msg);
			break;
	}
}

#endif /* #ifdef DEBUG */
