/**
 * @file       led.h
 * @brief      Led driver
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/29
 */

#include <io.h>
#include "led.h"
#include "servo.h"

/* ------------------------------------------------------------------------ */
/* static                                                                   */
/* ------------------------------------------------------------------------ */
static unsigned char ledMode_[LED_NBR];
static unsigned char ledTimer_=0;

void ledTimerCallback();

/* ------------------------------------------------------------------------ */
/* ledInit                                                                  */
/* ------------------------------------------------------------------------ */
void ledInit()
{
	unsigned char i=0;
	for(; i < LED_NBR; i++) {
		sbi(LED_IO_DIR, i + LED_FIRST_PIN);
		ledSetMode(i, LED_MODE_OFF);
	}

	servoRegisterPeriodicCB1(ledTimerCallback);
}

/* ------------------------------------------------------------------------ */
/* ledSetMode                                                               */
/* ------------------------------------------------------------------------ */
void ledSetMode(unsigned char ledId,
			    unsigned char mode)
{
	ledMode_[ledId] = mode;
	if (mode == LED_MODE_ON) {
		sbi(LED_IO_PORT, ledId + LED_FIRST_PIN);
	} else {
		cbi(LED_IO_PORT, ledId + LED_FIRST_PIN);
	}
}

/* ------------------------------------------------------------------------ */
/* ledTimerCallback                                                         */
/* ------------------------------------------------------------------------ */
void ledTimerCallback()
{
	unsigned char i=0;
	for(; i < LED_NBR; i++) {
		if ((ledTimer_ == 0) && 
		    ((ledMode_[i] == LED_MODE_BLINK_SLOW) || 
		     (ledMode_[i] == LED_MODE_BLINK_FAST))) {
			 sbi(LED_IO_PORT, i + LED_FIRST_PIN);
		} else if ((ledTimer_ == 25 || ledTimer_ == 75 || ledTimer_ == 125) && 
		           (ledMode_[i] == LED_MODE_BLINK_FAST)) {
			 cbi(LED_IO_PORT, i + LED_FIRST_PIN);
		} else if ((ledTimer_ == 50 || ledTimer_ == 100) && 
		           (ledMode_[i] == LED_MODE_BLINK_FAST)) {
			 sbi(LED_IO_PORT, i + LED_FIRST_PIN);
		} else if ((ledTimer_ == 75) && 
		           (ledMode_[i] == LED_MODE_BLINK_SLOW)) {
			 cbi(LED_IO_PORT, i + LED_FIRST_PIN);
		}
	}
	if ((++ledTimer_) > 150) ledTimer_ = 0;
}
