#include <interrupt.h>

#include "util.h"
#include "uart.h"
#include "adc.h"
#include "servo.h" 
#include "motor.h"
#include "bumper.h"
#include "led.h"
#include "wheel.h"
#include "solarDir.h"
#include "pinConfig.h"
#include "debug.h"

#define MAIN_MODE_FORWARD       0
#define MAIN_MODE_BACKWARD      1
#define MAIN_MODE_BEFORE_ROTATE 2
#define MAIN_MODE_ROTATE        3
#define MAIN_MODE_AFTER_ROTATE  4

#define MAIN_MAX_TIME_FORWARD  500
#define MAIN_MAX_TIME_BACKWARD 300
#define MAIN_MAX_TIME_ROTATE   200
#define MAIN_MAX_TIME_DELTA_ROTATE 50

static unsigned char  mainMode_ = MAIN_MODE_FORWARD;
static unsigned short mainTime_ = 0;
static unsigned char  mainTurnLeft_ = 0;

void mainPeriodicTask()
{
	mainTime_++;
	switch (mainMode_) {
	case MAIN_MODE_FORWARD:
		if ((mainTime_ > MAIN_MAX_TIME_FORWARD) 
		    || ((bumperGetData() & 0xF4) != 0xF4)) {
			mainMode_ = MAIN_MODE_BACKWARD;
			mainTime_ = 0;
			if ((bumperGetData() & 0x84) != 0x84) {
				// right bumpers
				mainTurnLeft_ = 0;
			} else {
				// left bumpers
				mainTurnLeft_ = 1;
			}
			wheelBackward();
			motorOn();
		}
		break;
	case MAIN_MODE_BACKWARD:
		if (mainTime_ > MAIN_MAX_TIME_BACKWARD) {
			mainMode_ = MAIN_MODE_BEFORE_ROTATE;
			mainTime_ = 0;
			motorOff();
			wheelStop();
			servoSetPosition(SERVO_WHEEL_REAR,  SERVO_WHEEL_REAR_ROTATE);
		}
		break;
	case MAIN_MODE_BEFORE_ROTATE:
		if (mainTime_ > MAIN_MAX_TIME_DELTA_ROTATE) {
			mainMode_ = MAIN_MODE_ROTATE;
			mainTime_ = 0;
			if (mainTurnLeft_){
				wheelRotateLeft();
			} else {
				wheelRotateRight();
			}
		}
		break;
	case MAIN_MODE_ROTATE:
		if (mainTime_ > MAIN_MAX_TIME_ROTATE) {
			mainMode_ = MAIN_MODE_AFTER_ROTATE;
			mainTime_ = 0;
			wheelStop();
			servoSetPosition(SERVO_WHEEL_REAR,  SERVO_WHEEL_REAR_FORWARD);
		}
		break;
	case MAIN_MODE_AFTER_ROTATE:
		if (mainTime_ > MAIN_MAX_TIME_DELTA_ROTATE) {
			mainMode_ = MAIN_MODE_FORWARD;
			mainTime_ = 0;
			wheelForward();
			motorOn();
		}
		break;
	}
}

/* ------------------------------------------------------------------------ */
/* mainInit                                                                 */
/* ------------------------------------------------------------------------ */
/* Initialize all components                                                */
/* ------------------------------------------------------------------------ */
void mainInit()
{
	ledInit();
	servoInit();
	motorInit();
	bumperInit();
	uartInit(UART_BAUD_RATE_9600);
	adcInit(ADC_SINGLE_CONVERSION, ADC_ENABLE_INTERRUPT);
	wheelInit();
	//solarInit();
	/* enable interrup */
	sei();

	mainMode_ = MAIN_MODE_FORWARD;
	mainTime_ = 0;
	wheelForward();
	motorOn();
	servoRegisterPeriodicCB2(mainPeriodicTask);
}

/* ------------------------------------------------------------------------ */
/* main                                                                     */
/* ------------------------------------------------------------------------ */
/* Program entry point                                                      */
/* ------------------------------------------------------------------------ */
int main(void)
{
	mainInit();
#ifdef DEBUG
	uartWrite('D');
	ledSetMode(0, LED_MODE_BLINK_FAST);
	servoSetPosition(0, 125);
	while(1) {
		debugParse(uartRead(UART_WAIT_RECEIVE));
	}
#else
	uartWrite('S');
	ledSetMode(0, LED_MODE_BLINK_SLOW);
	while(1) {
		utilDelay(10);
	}
#endif
	return 0;
}
