/**
 * @file       motor.c
 * @brief      Motor driver
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/25
 */

#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>
#include "motor.h"

/* ------------------------------------------------------------------------ */
/* static                                                                   */
/* ------------------------------------------------------------------------ */
static unsigned char motorSpeed_=0;
static unsigned char motorCounter_=0;

/* ------------------------------------------------------------------------ */
/* motorInit                                                                */
/* ------------------------------------------------------------------------ */
void motorInit()
{
	/* Set IO port as output */
	sbi(MOTOR_IO_DIR, MOTOR_PIN);
	motorOff();

	/* Set timer 0 speed : CLK/64 = 0.008ms/tick */
	outp(0x01, TCCR1B);  
	/* Set timer 0 value, when reach overflow, INTERRUPT(SIG_OVERFLOW0) is run.
	   It is run every ms*/
	outp(0xFF,TCNT1H);
	outp(0x80,TCNT1L);
	/* Enable timer 0 interrupt */
	sbi(TIMSK, TOIE1);
}

/* ------------------------------------------------------------------------ */
/* motorOn                                                                  */
/* ------------------------------------------------------------------------ */
void motorOn()
{
	motorSetSpeed(MOTOR_DEFAULT_SPEED);
}

/* ------------------------------------------------------------------------ */
/* motorOff                                                                 */
/* ------------------------------------------------------------------------ */
void motorOff()
{
	motorSetSpeed(0);
}

/* ------------------------------------------------------------------------ */
/* motorSetSpeed                                                            */
/* ------------------------------------------------------------------------ */
void motorSetSpeed(unsigned char speed)
{
	motorSpeed_ = speed;
}

/* ------------------------------------------------------------------------ */
/* INTERRUPT(SIG_OVERFLOW1)                                                 */
/* ------------------------------------------------------------------------ */
/* Interrupt when Timer 1 reached overflow                                  */
/* ------------------------------------------------------------------------ */
INTERRUPT(SIG_OVERFLOW1)  
{
	if ((motorCounter_ == motorSpeed_) && (motorSpeed_!=255)) {
		cbi(MOTOR_IO_PORT, MOTOR_PIN);
	}
	motorCounter_++;
	if ((motorCounter_ == 0) && (motorSpeed_ != 0)) {
		sbi(MOTOR_IO_PORT, MOTOR_PIN);
	}
	outp(0xFF,TCNT1H);
	outp(0x80,TCNT1L);
}
