/**
 * @file       solarDir.h
 * @brief      Solar detector that returns the direction of the sun
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/25
 */

#ifndef __SOLAR_DIR_H__
#define __SOLAR_DIR_H__

/** @brief Sun direction has just been lost and the detector is searching it */
#define SOLAR_DIR_UNKNOWN 0x8000
/** @brief The detector is in the shadow, it can't detect anything */
#define SOLAR_NOT_VISIBLE 0x4000

/**
 * @brief Initialize the solar dir detector
 */
void solarInit();

/**
 * @brief Return the direction of the sun
 * @return Return the direction of the sun [0..0xAA]
 *         SOLAR_DIR_UNKNOWN: sun direction has just been lost and the detector is searching it
 *         SOLAR_NOT_VISIBLE: the detector is in the shadow, it can't detect anything
 */
unsigned short solarGetDir();

/**
 * @brief Send debug informations on serial port
 */
void solarDump();

#endif /* __SOLAR_DIR_H__ */
